package main

import (
	"context"
	"fmt"
	"github.com/urfave/cli/v3"
	"io"
	"os"
	"strings"
	"time"

	"gitea.stevedudenhoeffer.com/steve/go-extractor/cmd/browser/pkg/browser"
	"gitea.stevedudenhoeffer.com/steve/go-extractor/sites/duckduckgo"
)

type DuckDuckGoFlags []cli.Flag

var Flags = DuckDuckGoFlags{
	&cli.StringFlag{
		Name:    "region",
		Aliases: []string{"r"},
	},
	&cli.StringFlag{
		Name:    "safesearch",
		Aliases: []string{"s"},
	},
}

func (f DuckDuckGoFlags) ToConfig(cmd *cli.Command) duckduckgo.Config {
	var res = duckduckgo.DefaultConfig

	if r := cmd.String("region"); r != "" {
		res.Region = r
	}

	if s := cmd.String("safesearch"); s != "" {
		switch s {
		case "on":
			res.SafeSearch = duckduckgo.SafeSearchOn
		case "moderate":
			res.SafeSearch = duckduckgo.SafeSearchModerate
		case "off":
			res.SafeSearch = duckduckgo.SafeSearchOff
		default:
			panic("invalid safe search value")
		}
	}

	return res
}

func deferClose(cl io.Closer) {
	if cl != nil {
		_ = cl.Close()
	}
}

func main() {
	var flags []cli.Flag

	flags = append(flags, browser.Flags...)
	flags = append(flags, Flags...)

	cli := &cli.Command{
		Name:  "duckduckgo",
		Usage: "Search DuckDuckGo",
		Flags: flags,
		Action: func(ctx context.Context, command *cli.Command) error {
			c := Flags.ToConfig(command)
			defer deferClose(nil)

			query := strings.TrimSpace(strings.Join(command.Args().Slice(), " "))

			if query == "" {
				return cli.Exit("usage: duckduckgo <query>", 1)
			}

			b, err := browser.FromCommand(ctx, command)
			defer deferClose(b)

			if err != nil {
				return fmt.Errorf("failed to create browser: %w", err)
			}

			search, err := c.OpenSearch(ctx, b, query)
			if err != nil {
				return fmt.Errorf("failed to open search: %w", err)
			}

			defer deferClose(search)

			res := search.GetResults()
			fmt.Println("Results:", res)

			err = search.LoadMore()
			if err != nil {
				return fmt.Errorf("failed to load more: %w", err)
			}

			time.Sleep(2 * time.Second)
			res = search.GetResults()
			fmt.Println("Results:", res)

			return nil
		},
	}

	err := cli.Run(context.Background(), os.Args)

	if err != nil {
		panic(err)
	}
}
